package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.domain.SysRoleDept;
import com.ruoyi.system.domain.SysRoleMenu;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.mapper.SysRoleDeptMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysRoleMenuMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysRoleService;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class SysRoleServiceImpl implements ISysRoleService {
   @Autowired
   private SysRoleMapper roleMapper;
   @Autowired
   private SysRoleMenuMapper roleMenuMapper;
   @Autowired
   private SysUserRoleMapper userRoleMapper;
   @Autowired
   private SysRoleDeptMapper roleDeptMapper;

   @DataScope(
      deptAlias = "d"
   )
   public List<SysRole> selectRoleList(SysRole role) {
      return this.roleMapper.selectRoleList(role);
   }

   public List<SysRole> selectRolesByUserId(Long userId) {
      List<SysRole> userRoles = this.roleMapper.selectRolePermissionByUserId(userId);
      List<SysRole> roles = this.selectRoleAll();
      Iterator var4 = roles.iterator();

      while(true) {
         while(var4.hasNext()) {
            SysRole role = (SysRole)var4.next();
            Iterator var6 = userRoles.iterator();

            while(var6.hasNext()) {
               SysRole userRole = (SysRole)var6.next();
               if (role.getRoleId() == userRole.getRoleId()) {
                  role.setFlag(true);
                  break;
               }
            }
         }

         return roles;
      }
   }

   public Set<String> selectRolePermissionByUserId(Long userId) {
      List<SysRole> perms = this.roleMapper.selectRolePermissionByUserId(userId);
      Set<String> permsSet = new HashSet();
      Iterator var4 = perms.iterator();

      while(var4.hasNext()) {
         SysRole perm = (SysRole)var4.next();
         if (StringUtils.isNotNull(perm)) {
            permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
         }
      }

      return permsSet;
   }

   public List<SysRole> selectRoleAll() {
      return ((SysRoleServiceImpl)SpringUtils.getAopProxy(this)).selectRoleList(new SysRole());
   }

   public List<Long> selectRoleListByUserId(Long userId) {
      return this.roleMapper.selectRoleListByUserId(userId);
   }

   public SysRole selectRoleById(Long roleId) {
      return this.roleMapper.selectRoleById(roleId);
   }

   public boolean checkRoleNameUnique(SysRole role) {
      Long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
      SysRole info = this.roleMapper.checkRoleNameUnique(role.getRoleName());
      return !StringUtils.isNotNull(info) || info.getRoleId() == roleId;
   }

   public boolean checkRoleKeyUnique(SysRole role) {
      Long roleId = StringUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
      SysRole info = this.roleMapper.checkRoleKeyUnique(role.getRoleKey());
      return !StringUtils.isNotNull(info) || info.getRoleId() == roleId;
   }

   public void checkRoleAllowed(SysRole role) {
      if (StringUtils.isNotNull(role.getRoleId()) && role.isAdmin()) {
         throw new ServiceException("不允许操作超级管理员角色");
      }
   }

   public void checkRoleDataScope(Long... roleIds) {
      if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
         Long[] var2 = roleIds;
         int var3 = roleIds.length;

         for(int var4 = 0; var4 < var3; ++var4) {
            Long roleId = var2[var4];
            SysRole role = new SysRole();
            role.setRoleId(roleId);
            List<SysRole> roles = ((SysRoleServiceImpl)SpringUtils.getAopProxy(this)).selectRoleList(role);
            if (StringUtils.isEmpty(roles)) {
               throw new ServiceException("没有权限访问角色数据！");
            }
         }
      }

   }

   public int countUserRoleByRoleId(Long roleId) {
      return this.userRoleMapper.countUserRoleByRoleId(roleId);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int insertRole(SysRole role) {
      this.roleMapper.insertRole(role);
      return this.insertRoleMenu(role);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int updateRole(SysRole role) {
      this.roleMapper.updateRole(role);
      this.roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
      return this.insertRoleMenu(role);
   }

   public int updateRoleStatus(SysRole role) {
      return this.roleMapper.updateRole(role);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int authDataScope(SysRole role) {
      this.roleMapper.updateRole(role);
      this.roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
      return this.insertRoleDept(role);
   }

   public int insertRoleMenu(SysRole role) {
      int rows = 1;
      List<SysRoleMenu> list = new ArrayList();
      Long[] var4 = role.getMenuIds();
      int var5 = var4.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         Long menuId = var4[var6];
         SysRoleMenu rm = new SysRoleMenu();
         rm.setRoleId(role.getRoleId());
         rm.setMenuId(menuId);
         list.add(rm);
      }

      if (list.size() > 0) {
         rows = this.roleMenuMapper.batchRoleMenu(list);
      }

      return rows;
   }

   public int insertRoleDept(SysRole role) {
      int rows = 1;
      List<SysRoleDept> list = new ArrayList();
      Long[] var4 = role.getDeptIds();
      int var5 = var4.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         Long deptId = var4[var6];
         SysRoleDept rd = new SysRoleDept();
         rd.setRoleId(role.getRoleId());
         rd.setDeptId(deptId);
         list.add(rd);
      }

      if (list.size() > 0) {
         rows = this.roleDeptMapper.batchRoleDept(list);
      }

      return rows;
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int deleteRoleById(Long roleId) {
      this.roleMenuMapper.deleteRoleMenuByRoleId(roleId);
      this.roleDeptMapper.deleteRoleDeptByRoleId(roleId);
      return this.roleMapper.deleteRoleById(roleId);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int deleteRoleByIds(Long[] roleIds) {
      Long[] var2 = roleIds;
      int var3 = roleIds.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Long roleId = var2[var4];
         this.checkRoleAllowed(new SysRole(roleId));
         this.checkRoleDataScope(roleId);
         SysRole role = this.selectRoleById(roleId);
         if (this.countUserRoleByRoleId(roleId) > 0) {
            throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
         }
      }

      this.roleMenuMapper.deleteRoleMenu(roleIds);
      this.roleDeptMapper.deleteRoleDept(roleIds);
      return this.roleMapper.deleteRoleByIds(roleIds);
   }

   public int deleteAuthUser(SysUserRole userRole) {
      return this.userRoleMapper.deleteUserRoleInfo(userRole);
   }

   public int deleteAuthUsers(Long roleId, Long[] userIds) {
      return this.userRoleMapper.deleteUserRoleInfos(roleId, userIds);
   }

   public int insertAuthUsers(Long roleId, Long[] userIds) {
      List<SysUserRole> list = new ArrayList();
      Long[] var4 = userIds;
      int var5 = userIds.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         Long userId = var4[var6];
         SysUserRole ur = new SysUserRole();
         ur.setUserId(userId);
         ur.setRoleId(roleId);
         list.add(ur);
      }

      return this.userRoleMapper.batchUserRole(list);
   }
}
